% Fig 8-26
clear all ;% clear all variables
close all

%============== Define global variables ===================================================
global BitRate Bits ChannelSpacing ChannelNumber SampleRate DataLength samplesperbit t_range  f_sym 

%Ref_Frequency = 193.1e12; %[Hz] Reference optical frequency, usually center frequency of simulations

BitRate = 10e9;  
Bits2Power = 10;
Bits = 2^Bits2Power;
TimeWindow = Bits/BitRate;
ChannelNumber = 5;
ChannelSpacing = 100e9;   %[Hz]

SampleRateMargin = 5;  %MarginBandwidth = SampleRateMargin*ChannelSpacing
SampleRate = BitRate*SampleRateFactor(SampleRateMargin); %[Hz]
DataLength = TimeWindow*SampleRate;
samplesperbit = SampleRate/BitRate; % data number per bit

% time, optical frequency scale
t_range = 0:1/SampleRate:1/SampleRate*(DataLength-1);
f_sym = -.5/TimeWindow*(DataLength):1/TimeWindow:.5/TimeWindow*(DataLength-1);

%==========================================================================================

%=================== Define Simulation Blocks =============================================

%-------------- Laser Array -----------------------%
LaserNumber = ChannelNumber;
LaserSpacing = ChannelSpacing; %[Hz] Equal channel spacing

%LaserArrayCenterFreq = Ref_Frequency; %[Hz]

%--------------- Array: External Modulation ------------%
%=== Physical Parameters=======
PreSpaces = 2;  %number of leading zero bits
PostSpaces = 2; %number of trailing zero bits

RiseTime = 1/(4*BitRate); % 
UnmodulatedPower =  20e-3; %[W] %Fixed. Power level to fiber will be controlled by booster amp.
Linewidth = 0*50e6; %[Hz]
ModulationIndex = 1; %

[E_LaserArray,info_bits] = Array_TxModAmplitude_NRZ(UnmodulatedPower,Linewidth,...
          ModulationIndex,PreSpaces,PostSpaces,RiseTime,LaserNumber,LaserSpacing);
%------------------------------------------------------------------------------%
                  
%-------------------(N by 1) Mux ----------------------------%                 

E_Array_Muxed = WDM_Mux_Ideal(E_LaserArray);

%-------------------------------------------------------------%
% Add Noise to Control OSNR   
% NoiseCenterFrequency  = Ref_Frequency;
OSNR_target = 30; %[dB] Target OSNR
% Res_FilterWidth is the noise bandwidth to calculate noise power.
% Res_FilterWidth must be sufficiently wide to accurately measure the power level of the signal
% Ex) the signal power of 10Gbit/s with zero chirp will measure 0.8dB low with a 0.1nm Resolution BW.
% To addequately display OSNR, the 'Res_Filter Bandwidth of Optical SpectrumAnalyzer' should be
% equal to Res_FilterWidth for noise power.

Res_FilterWidth = BitRate;%  
NoisePower = (UnmodulatedPower/2)*10^(-OSNR_target/10);%[W] UnmodulatedPower/2 = modulated power in NRZ 
NoisePowerDensity = NoisePower/Res_FilterWidth; %[W/Hz]
noise_add = NoiseWhiteG_Optical(NoisePowerDensity);

E_Array_MuxedOut = E_Array_Muxed + noise_add;    

% Define OpticalSpectrumAnalyzer parameters
OpticalSpectrumAnalyzer(E_Array_MuxedOut,Res_FilterWidth,'Rectangular');

% Calculation of OSNR in [dB] 
OsnrdB1 = OSNR_Meter(E_Array_MuxedOut,Res_FilterWidth,Res_FilterWidth,ChannelSpacing/2,-2*ChannelSpacing,'Rectangular')
OsnrdB = OSNR_MeterArray(E_Array_MuxedOut,Res_FilterWidth,Res_FilterWidth,'Rectangular')
